﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Diagnostics;

namespace RobobinApp.Networking
{
    public class WifiManager : IWifiService
    {
        private UdpClient _udpClient;
        private const int BroadcastPort = 5005;
        private bool _isConnected = false; // Flag to indicate connection status
        private CancellationTokenSource _cancellationTokenSource; // For stopping the UDP listener

        public WifiManager()
        {
            _udpClient = new UdpClient(BroadcastPort);
            _cancellationTokenSource = new CancellationTokenSource();
        }

        public async Task StartListening()
        {
            while (!_isConnected) // Continue listening until connected
            {
                Debug.WriteLine("Waiting for broadcast...");
                var result = await _udpClient.ReceiveAsync();
                string message = Encoding.ASCII.GetString(result.Buffer);

                if (message == "ROBOBIN_PRESENT")
                {
                    Debug.WriteLine("Detected Robobin presence from: " + result.RemoteEndPoint);
                    SendConnectMessage(result.RemoteEndPoint.Address.ToString());
                }
            }

            // Stop listening if connected
            Debug.WriteLine("Stopping UDP listener.");
            _udpClient.Close();
        }

        public void SendConnectMessage(string ipAddress)
        {
            if (_isConnected)
            {
                Debug.WriteLine("Already connected. No need to send another connect message.");
                return;
            }

            var endPoint = new IPEndPoint(IPAddress.Parse(ipAddress), 5006);
            byte[] message = Encoding.ASCII.GetBytes("CONNECT");
            _udpClient.Send(message, message.Length, endPoint);
            Debug.WriteLine($"Sent connect message to: {ipAddress}");

            Task.Run(() => ConnectToTcpServer(endPoint));
        }
        public async Task SendPingMessage(TcpClient tcpClient)
        {
            if (!_isConnected)
            {
                Debug.WriteLine("Not connected. Cannot send ping message.");
                return;
            }

            try
            {
                NetworkStream stream = tcpClient.GetStream();
                byte[] pingMessage = Encoding.ASCII.GetBytes("PING");
                await stream.WriteAsync(pingMessage, 0, pingMessage.Length);
                Debug.WriteLine("Sent PING message to Robobin.");
            }
            catch (Exception ex)
            {
                Debug.WriteLine($"Failed to send PING message: {ex.Message}");
            }
        }

        private async Task ConnectToTcpServer(IPEndPoint endPoint)
        {
            using (TcpClient tcpClient = new TcpClient())
            {
                try
                {
                    await tcpClient.ConnectAsync(endPoint.Address, endPoint.Port);
                    Debug.WriteLine("Connected to Robobin via TCP.");

                    _isConnected = true;

                    // Keep the connection open to send PING messages periodically
                    while (_isConnected)
                    {
                        await SendPingMessage(tcpClient);
                        await Task.Delay(5000); // Send a ping every 5 seconds
                    }
                }
                catch (Exception ex)
                {
                    Debug.WriteLine($"TCP connection failed: {ex.Message}");
                }
            }

            _cancellationTokenSource.Cancel(); // Cancel the token to stop the UDP listener
        }

    }
}
